/*
Dozen Days of Words episode 4: GrowingWords
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.growing
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.utils.getTimer;

	import com.blazinggames.dozendays.TextButton;
	import com.blazinggames.dozendays.YesNoDialog;
	import com.blazinggames.dozendays.words.LetterTile;

	public class GrowingGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const SKIP_TEXT:String = "Skip";
		public static const DONE_TEXT:String = "Title!";
		public static const EMPTY_TEXT:String = "-";
		public static const WON_TEXT:String = "You Have WON!";
		public static const LOST_TEXT:String = "You Have Lost!";

		public static const TILE_SIZE:int = 40;
		public static const NUM_ROWS:int = 12;
		public static const NUM_COLS:int = 12;
		
		public static const BASE_STARTING_LETTERS:int = 5;
		public static const STARING_LETTERS_MULTIPLIER:int = 5;
		public static const POINTS_PER_LETTER:int = 5;

		public static const BACKGROUND_COLOR:uint = 0x000050;
		public static const TILE_COLOR:uint = 0x500050;
		public static const BORDER_COLOR:uint = 0x000030;
		public static const LETTER_COLOR:uint = 0xC000C0;
		public static const TILE_COLOR_DIM:uint = 0x200020;
		public static const LETTER_COLOR_DIM:uint = 0x500050;
		public static const HIGHLIGHT_COLOR:uint = 0xFF0050;
		public static const WORD_COLOR:uint = 0x00C000;

// ***** VARIABLES **********************************************************

		protected var _growing:GrowingWords;
		protected var _doneButton:TextButton;
		protected var _skipButton:TextButton;
		protected var _tiles:Array;
		protected var _backdrop:TextField;
		protected var _haveWon:TextField;
		protected var _scoreText:TextField;
		
		protected var _difficulty:int;
		protected var _currentLevel:int;
		protected var _score:int;
		protected var _growingList:GrowingList;
		protected var _selection:LetterTile;
		protected var _levelCompleteButton:TextButton;

// ***** CONSTRUCTOR ********************************************************

		public function GrowingGame(growing:GrowingWords)
		{
			var cntrRow:int, cntrCol:int, rowLength:int, tileX:int, tileY:int;
			_growing = growing;
			_selection = null;

			_backdrop = new TextField();
			_backdrop.multiline = true;
			_backdrop.width = 640;
			_backdrop.height = 480;
			_backdrop.textColor = 0xC000C0;
			_backdrop.background = true;
			_backdrop.backgroundColor = BACKGROUND_COLOR;
			addChild(_backdrop);
			
			_tiles = new Array();
			var indx:int = 0;
			var wordFormat:TextFormat = new TextFormat(null, 20, WORD_COLOR, null, null, null, null, null, TextFormatAlign.CENTER);
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				tileX = 0;
				tileY = cntrRow * TILE_SIZE;
				for (cntrCol = 0; cntrCol < NUM_COLS; ++cntrCol) {
					_tiles[indx] = new LetterTile(TILE_SIZE,TILE_SIZE, EMPTY_TEXT);
					_tiles[indx].x = tileX;
					_tiles[indx].y = tileY;
					_tiles[indx].setColors(LETTER_COLOR, TILE_COLOR);
					_tiles[indx].setBorder(true, BORDER_COLOR);
					_tiles[indx].addEventListener(LetterTile.TILE_SELECTED_EVENT, handleTileSelection);
					addChild(_tiles[indx]);

					tileX += TILE_SIZE;
					++indx;
				}
				
			}

			_skipButton = new TextButton(SKIP_TEXT, 75, 40);
			_skipButton.x = 480;
			_skipButton.y = 0;
			addChild(_skipButton);
			_skipButton.addEventListener(MouseEvent.MOUSE_UP, handleSkip);

			_doneButton = new TextButton(DONE_TEXT, 80, 40);
			_doneButton.x = 560;
			_doneButton.y = 0;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			_growingList = new GrowingList();
			_growingList.x = 490;
			_growingList.y = 50;
			addChild(_growingList);

			var winFormat:TextFormat = new TextFormat(null, 90, 0xFFFF00, null, null, null, null, null, TextFormatAlign.CENTER);
			_haveWon = new TextField();
			_haveWon.width = 640;
			_haveWon.height = 150;
			_haveWon.y = 200;
			_haveWon.defaultTextFormat = winFormat;
			_haveWon.text = WON_TEXT;
			_haveWon.visible = false;
			addChild(_haveWon);

			_levelCompleteButton = new TextButton("?", 640, 40);
			_levelCompleteButton.x = 0;
			_levelCompleteButton.y = 200;
			_levelCompleteButton.visible = false;
			addChild(_levelCompleteButton);
			_levelCompleteButton.addEventListener(MouseEvent.MOUSE_UP, handleNextLevel);

			var scoreFormat:TextFormat = new TextFormat(null, 20, 0xF0F0FF, null, null, null, null, null, TextFormatAlign.CENTER);
			_scoreText = new TextField();
			_scoreText.width = 160;
			_scoreText.height = 25;
			_scoreText.x = 480;
			_scoreText.y = 455;
			_scoreText.defaultTextFormat = scoreFormat;
			_scoreText.text = "0";
			addChild(_scoreText);
		}

// ***** Functions - Puzzle Handling ****************************************

		public function startGame(playmode:int = 3):void
		{
			
			_difficulty = playmode;
			_score = 0;
			_scoreText.text = "0";
			startLevel(1);
		}

		public function addLetter(s:String):void
		{
			var placedLetter:Boolean = false;
			
			// first, select random location to place letter
			var targetSpot:int = Math.floor(Math.random() * _tiles.length);
			var indx:int = targetSpot;

			// search from location to end for first blank
			do {
				if (_tiles[indx].getLetter() == EMPTY_TEXT) {
					_tiles[indx].setLetter(s);
					placedLetter = true;
				}
				++indx;
			} while (( !placedLetter) && (indx < _tiles.length));

			// search from beginning to location for
			indx = 0;
			while (( !placedLetter) && (indx < targetSpot)) {
				if (_tiles[indx].getLetter() == EMPTY_TEXT) {
					_tiles[indx].setLetter(s);
					placedLetter = true;
				}
				++indx;
			}
			
			if ( !placedLetter) {
				_haveWon.text = LOST_TEXT;
				_haveWon.visible = true;
			}
		}

		private function startLevel(n:int):void
		{
			var cntr:int;
			var numStaringLetters:int = BASE_STARTING_LETTERS + _difficulty * STARING_LETTERS_MULTIPLIER;

			_levelCompleteButton.visible = false;
			_haveWon.visible = false;
			_currentLevel = n;
			var nextLevel = ((n -1) % 5) + 1;
			_growingList.startLevel(nextLevel);
			// clear the game board
			for (cntr = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setLetter(EMPTY_TEXT);
			
			// add starting letters
			for (cntr = 0; cntr < numStaringLetters; ++cntr)
				addLetter(_growingList.getNextLetter());
		}

// ***** Functions - Event Handling *****************************************

		private function handleSkip(evt:Event):void
		{
			endTurn();
		}

		private function handleExit(evt:Event):void
		{
			_growing.changeMode(GrowingWords.MODE_TITLE);
		}

		private function handleNextLevel(evt:Event):void
		{
			++_currentLevel;
			if ((_currentLevel % 5) == 1)
				++_difficulty;
			startLevel(_currentLevel);
		}

		private function handleTileSelection(evt:Event):void
		{
			var cntr:int;
			var selected:LetterTile = LetterTile(evt.target);
			var indx:int = -1;

			// figure out which tile was clicked on
			for (cntr = 0; cntr < _tiles.length; ++cntr) {
				if (selected == _tiles[cntr])
					indx = cntr;
			}
			if (indx < 0)
				return;
				
			// base handling of clicked tile on selection status
			if (_selection == null) {
				if (selected.getLetter() != EMPTY_TEXT) {
					_selection = selected;
					selected.setColors(LETTER_COLOR_DIM, TILE_COLOR_DIM);
				}
			} else {
				if (selected.getLetter() != EMPTY_TEXT) {
					_selection.setColors(LETTER_COLOR, TILE_COLOR);
					_selection = selected;
					selected.setColors(LETTER_COLOR_DIM, TILE_COLOR_DIM);
				} else {
					var s:String = _selection.getLetter();
					_selection.setLetter(EMPTY_TEXT);
					_selection.setColors(LETTER_COLOR, TILE_COLOR);
					selected.setLetter(s);
					endTurn();
				}
			}
		}

// ***** Functions - game handling *******************************************

		public function endTurn():void {
			var cntrRow:int, cntrCol:int, cntr:int;
			var wordLength:int = _growingList.getWordLength();
			var s:String;
			var scored:Boolean = false;
			
			// check for valid words and remove them
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				for (cntrCol = 0; cntrCol < NUM_COLS; ++cntrCol) {
					// horizontal check
					s = getLetter(cntrCol, cntrRow);
					for (cntr = 1; cntr < wordLength; ++cntr)
						s += getLetter(cntrCol + cntr, cntrRow);
					trace ("Horizontal check for " + s);
					if (_growingList.wasWordInList(s)) {
						for (cntr = 0; cntr < wordLength; ++cntr)
							setLetter(cntrCol + cntr, cntrRow, EMPTY_TEXT);
						scored = true;
					}

					// vertical check
					s = getLetter(cntrCol, cntrRow);
					for (cntr = 1; cntr < wordLength; ++cntr)
						s += getLetter(cntrCol, cntrRow + cntr);
					trace ("Vertical check for " + s);
					if (_growingList.wasWordInList(s)) {
						for (cntr = 0; cntr < wordLength; ++cntr)
							setLetter(cntrCol, cntrRow + cntr, EMPTY_TEXT);
						scored = true;
					}
				}
			}
			if (scored) {
				var temp:int = wordLength * POINTS_PER_LETTER * _difficulty;
				_score += temp;
				_scoreText.text = "" + _score;
				// see if level completed
				if (_growingList.getWordsRemaining() == 0) {
					_levelCompleteButton.setLabel("Level " + _currentLevel + " completed!");
					_levelCompleteButton.visible = true;
				}
				return;

			}
			
			// add new letters to board
			for (cntr = 0; cntr < _difficulty; ++cntr)
				addLetter(_growingList.getNextLetter());
		}
		
		public function getLetter(tx:int, ty:int):String
		{
			if ( (tx < 0) || (tx >= NUM_COLS) || (ty < 0) || (ty >= NUM_ROWS) )
				return "*";
			var indx:int = ty * NUM_COLS + tx;
			return _tiles[indx].getLetter();
		}

		public function setLetter(tx:int, ty:int, s:String):void
		{
			if ( (tx < 0) || (tx >= NUM_COLS) || (ty < 0) || (ty >= NUM_ROWS) )
				return ;
			var indx:int = ty * NUM_COLS + tx;
			_tiles[indx].setLetter(s);
		}
	}
}
